<?php

/**
 * @file
 * Administrator form functions for Deep Survey Skip module.
 *
 * Admin form functions.
 *
 * Contains all the functionality of the forms from the admin pages that need
 * global scope.
 */

/**
 * This is the survey skip's implementation of the node specific edit function.
 *
 * @param int $deepsurvey_item_nid
 *   Node id of the item displaying it's child items.
 *
 * @param string $node_type
 *   The type of the item displaying it's child items.
 *
 * @param string $current_path
 *   The current path to this page, used to build links to the child nodes.
 *
 * @param string $last_page
 *   The path to the previous path for redirects, needed because of the
 *   limitations in drupal menu paths.
 *
 * @return array
 *   Form.
 */
function deepsurvey_skip_deepsurvey_edit_form($deepsurvey_item_nid, $node_type, $current_path, $last_page) {

  // Counts the loops and hence the order of the items.
  $order = 0;

  // Work around for AHAH bug.
  $form = array(
    '#cache' => TRUE,
    '#action' => url($current_path),
  );

  $form['start'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
  );
  $form['start']['new'] = array(
    '#value' => l(t('add new rules to the start of the skip'),
    $current_path . '/skip/0'),
    '#theme' => 'item',
  );
  $form['table'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#theme' => 'deepsurvey_skip_edit_table',
    '#prefix' => '<div id="deepsurvey-skip-edit">',
    '#suffix' => '</div>',
  );

  // Create a list of possible nid codes for response and target.
  $all_questions_list = array();
  $parent_node = substr(strrchr($last_page, "/"), 1);
  $current_question = db_fetch_object(db_query('
    SELECT type FROM {node}
    WHERE nid = %d', $parent_node));
  $parent_type = $current_question->type;
  $question_list = array();
  if ($parent_type == 'deepsurvey') {
    $all_questions = db_query('SELECT n.nid, n.title, r.item_order, n.type
      FROM {node} n
      INNER JOIN {deepsurvey_node_relationship} r
      ON n.nid = r.item_nid
      WHERE r.deepsurvey_nid = %d 
      ORDER BY r.item_order',
      $parent_node);
    $skip_order = db_fetch_object(db_query('
      SELECT item_order
      FROM {deepsurvey_node_relationship}
      WHERE item_nid = %d', $deepsurvey_item_nid));
  }
  elseif ($parent_type) {
    $all_questions = db_query('SELECT n.nid, n.title, r.item_order, n.type
      FROM {node} n
      INNER JOIN {deepsurvey_%s_node_relationship} r
      ON n.nid = r.item_nid
      WHERE r.deepsurvey_nid = %d
      ORDER BY r.item_order',
      $parent_type, $parent_node);
    $skip_order = db_fetch_object(db_query('
      SELECT item_order
      FROM {deepsurvey_%s_node_relationship}
      WHERE item_nid = %d', $parent_type, $deepsurvey_item_nid));
  }
  while ($current_question = db_fetch_object($all_questions)) {
    $question_list[$current_question->nid] = $current_question->title;
    if (($current_question->item_order < $skip_order->item_order) && $current_question->type == 'deepsurvey_question') {
      $response_question_list[$current_question->nid] = $current_question->title;
    }
    elseif ($current_question->item_order > $skip_order->item_order) {
      $target_list[$current_question->nid] = $current_question->title;
    }
  }

  if (empty($target_list)) {
    $target_list = $question_list;
  }

  // Create a list of all possible questions.
  $all_questions_list_query = db_query('SELECT nid, title
    FROM {node} 
    WHERE type = "deepsurvey_question"');
  while ($current_question_list = db_fetch_object($all_questions_list_query)) {
    $all_questions_list[$current_question_list->nid] = $current_question_list->title;
  }

  // Loop through skip rules to build form.
  $result = db_query('
    SELECT item_order, response_nid, operation, row_code,
    column_code, drop_down_code, response, target_nid
    FROM {deepsurvey_skip_responses}
    WHERE deepsurvey_nid = %d
    ORDER BY item_order', $deepsurvey_item_nid);
  while ($nodes = db_fetch_object($result)) {

    $order++;

    // Retrieve drop down box options based upon response question.
    // Row.
    $row_list = array();
    $all_row = db_query('SELECT a.response, a.code
      FROM {deepsurvey_answer_list_responses} a
      INNER JOIN {deepsurvey_question_node_relationship} q
      ON a.deepsurvey_nid = q.item_nid
      WHERE q.deepsurvey_nid = %d AND q.axis = 1', $nodes->response_nid);
    while ($current_row = db_fetch_object($all_row)) {
      $row_list[$current_row->code] = strip_tags($current_row->code . ': ' . $current_row->response);
    }

    // Column.
    $column_list = array();
    $all_column = db_query('SELECT a.response, a.code
      FROM {deepsurvey_answer_list_responses} a
      INNER JOIN {deepsurvey_question_node_relationship} q
      ON a.deepsurvey_nid = q.item_nid
      WHERE q.deepsurvey_nid = %d AND q.axis = 2', $nodes->response_nid);
    while ($current_column = db_fetch_object($all_column)) {
      $column_list[$current_column->code] = strip_tags($current_column->code . ': ' . $current_column->response);
    }

    // Drop down.
    $drop_down_list = array();
    $all_drop_down = db_query('SELECT a.response, a.code
      FROM {deepsurvey_answer_list_responses} a
      INNER JOIN {deepsurvey_question_node_relationship} q
      ON a.deepsurvey_nid = q.item_nid
      WHERE q.deepsurvey_nid = %d AND q.axis = 3', $nodes->response_nid);
    while ($current_drop_down = db_fetch_object($all_drop_down)) {
      $drop_down_list[$current_drop_down->code] = strip_tags($current_drop_down->code . ': ' . $current_drop_down->response);
    }

    // Check for invalid values due to changes in the response question.
    if ((empty($row_list) && $nodes->row_code) ||(empty($column_list) && $nodes->column_code) || (empty($drop_down_list) && $nodes->drop_down_code)) {
      drupal_set_message(t('Invalid rules due to a changes in the "skip when item" detected - click on "Done" to fix'), 'warning');
    }

    // If the target list is empty allow any question to be selected.
    if (empty($target_list)) {
      $target_list = $all_questions_list;
    }

    $form['table'][$nodes->item_order]['order'] = array(
      '#value' => $order,
    );
    $form['table'][$nodes->item_order]['response title'] = array(
      '#type' => 'select',
      '#default_value' => $nodes->response_nid,
      '#options' => !is_null($response_question_list[$nodes->response_nid]) ? $response_question_list : array($nodes->response_nid => $all_questions_list[$nodes->response_nid]) + $response_question_list,
      '#parents' => array('table', t($nodes->item_order), 'response title'),
      '#ahah' => array(
        'path' => 'deepsurvey_skip_edit_ahah',
        'wrapper' => 'deepsurvey-skip-edit',
        'method' => 'replace',
        'event' => 'change',
      ),
    );
    $form['table'][$nodes->item_order]['operation'] = array(
      '#type' => 'select',
      '#default_value' => $nodes->operation,
      '#options' => array(
        DEEPSURVEY_SKIP_LT => t('<'),
        DEEPSURVEY_SKIP_EQ => t('='),
        DEEPSURVEY_SKIP_GT => t('>'),
      ),
      '#parents' => array('table', t($nodes->item_order), 'operation'),
    );
    $form['table'][$nodes->item_order]['row code'] = array(
      '#type' => 'select',
      '#maxlength' => 6,
      '#default_value' => empty($row_list) ? '' : $nodes->row_code,
      '#options' => array('' => 'any') + $row_list,
      '#disabled' => empty($row_list),
      '#parents' => array('table', t($nodes->item_order), 'row code'),
    );
    $form['table'][$nodes->item_order]['col code'] = array(
      '#type' => 'select',
      '#maxlength' => 6,
      '#default_value' => empty($column_list) ? '' : $nodes->column_code,
      '#options' => array('' => 'any') + $column_list,
      '#disabled' => empty($column_list),
      '#parents' => array('table', t($nodes->item_order), 'col code'),
    );
    $form['table'][$nodes->item_order]['drop down'] = array(
      '#type' => 'select',
      '#default_value' => empty($drop_down_list) ? '' : $nodes->drop_down_code,
      '#options' => array('' => 'any') + $drop_down_list,
      '#disabled' => empty($drop_down_list),
      '#parents' => array('table', t($nodes->item_order), 'drop down'),
    );
    $form['table'][$nodes->item_order]['response'] = array(
      '#type' => 'textfield',
      '#size' => 20,
      '#default_value' => $nodes->response,
      '#parents' => array('table', t($nodes->item_order), 'response'),
    );
    $form['table'][$nodes->item_order]['target title'] = array(
      '#type' => 'select',
      '#default_value' => $nodes->target_nid,
      '#options' => !is_null($target_list[$nodes->target_nid]) ? array('' => '') + $target_list : array('' => '', $nodes->target_nid => $all_questions_list[$nodes->target_nid]) + $target_list,
      '#parents' => array('table', t($nodes->item_order), 'target title'),
    );
    $form['table'][$nodes->item_order]['reorder box'] = array(
      '#type' => 'textfield',
      '#size' => 4,
      '#maxlength' => 4,
      '#parents' => array('table', t($nodes->item_order), 'reorder box'),
    );
    $form['table'][$nodes->item_order]['remove box'] = array(
      '#type' => 'checkbox',
      '#parents' => array('table', t($nodes->item_order), 'remove box'),
    );
    $form['table'][$nodes->item_order]['add new'] = array(
      '#value' => l(t('add new rules here'),
      $current_path . '/skip/' . $nodes->item_order),
    );
  };

  $form['last page'] = array(
    '#type' => 'value',
    '#value' => $last_page,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Done'),
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Back'),
  );

  return $form;
}

/**
 * Submit function for survey skip edit form.
 */
function deepsurvey_skip_deepsurvey_edit_form_submit($form, &$form_state) {
  deepsurvey_edit_process_skip($form, $form_state,
    'deepsurvey_answer_list_responses');
}


/**
 * Validate function for survey skip edit form.
 */
function deepsurvey_skip_deepsurvey_edit_form_validate($form, &$form_state) {
  deepsurvey_edit_validate_skip($form, $form_state);
}

/**
 * Processes the edit form.
 *
 * This is the main function where the processing of the edit form takes place,
 * it has been disentangled from the node specific function so other deep
 * survey modules can use it.
 *
 * @param array $form
 *   Standard form submit function parameter.
 *
 * @param array $form_state
 *   Standard form submit function parameter.
 *
 * @param string $node_table_properties
 *   String containing the name of the particular node properties table for
 *   this type.
 *
 * @return array
 *   Form array.
 */
function deepsurvey_edit_process_skip($form, &$form_state, $node_table_properties) {

  if ($form_state['clicked_button']['#value'] == t('Done')) {
    // Flag any reorder boxes being set.
    $local_flag = FALSE;
    $reset_flag = FALSE;
    $local_order = 1;
    $local_prev = 0;
    $local_offset = 0;
    $new_id = 0;
    // Item id of the reordered items.
    $new_order = array();
    // Relationship id of the items not being reordered.
    $old_order = array();
    reset($form_state['values']['table']);

    while (list($key, $form_order) = each($form_state['values']['table'])) {
      if ($form_state['values']['table'][$key]['remove box'] == TRUE) {
        db_query('
          DELETE FROM {deepsurvey_skip_responses}
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $key, $form_state['values']['nid']);
      }
      elseif ($form_state['values']['table'][$key]['reorder box'] != '') {
        // Remove item from current order.
        db_query('
          DELETE FROM {deepsurvey_skip_responses}
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $key, $form_state['values']['nid']);
        $new_order[$form_state['values']['table'][$key]['reorder box']] = array(
          $form_state['values']['table'][$key]['response title'],
          $form_state['values']['table'][$key]['operation'],
          $form_state['values']['table'][$key]['row code'],
          $form_state['values']['table'][$key]['col code'],
          $form_state['values']['table'][$key]['drop down'],
          $form_state['values']['table'][$key]['response'],
          $form_state['values']['table'][$key]['target title'],
        );
        $local_flag = TRUE;
      }
      else {
        $old_order[$local_order] = $key;
        $local_order++;
        // @todo Just update everything for now, optimise later if need be.
        db_query('
          UPDATE {deepsurvey_skip_responses}
          SET response_nid = %d, operation = %d, row_code = %d,
          column_code = %d, drop_down_code = %d, response = "%s", target_nid = %d
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $form_state['values']['table'][$key]['response title'],
          $form_state['values']['table'][$key]['operation'],
          $form_state['values']['table'][$key]['row code'],
          $form_state['values']['table'][$key]['col code'],
          $form_state['values']['table'][$key]['drop down'],
          $form_state['values']['table'][$key]['response'],
          $form_state['values']['table'][$key]['target title'], $key,
          $form_state['values']['nid']);
        if ($form_state['values']['table'][$key]['row code'] == '') {
          db_query('
          UPDATE {deepsurvey_skip_responses}
          SET row_code = NULL
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $key, $form_state['values']['nid']);
        }
        if ($form_state['values']['table'][$key]['col code'] == '') {
          db_query('
          UPDATE {deepsurvey_skip_responses}
          SET column_code = NULL
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $key, $form_state['values']['nid']);
        }
        if ($form_state['values']['table'][$key]['drop down'] == '') {
          db_query('
          UPDATE {deepsurvey_skip_responses}
          SET drop_down_code = NULL
          WHERE item_order = %d AND deepsurvey_nid = %d',
          $key, $form_state['values']['nid']);
        }
      }
    }

    // We now have an array of the survey item order, one with the order of the
    // items whose order was not changed and one with the changed order items.
    // Now compare the two and make changes to the database where necessary.
    $local_order = 1;
    if ($local_flag) {
      while ($old_order[$local_order - $local_offset] != FALSE || $new_order[$local_order] != FALSE) {
        if ($old_order[$local_order - $local_offset] != FALSE && $new_order[$local_order] != FALSE) {
          if ($local_prev >= $old_order[$local_order - $local_offset] + 1) {
            $loop_prev = $local_prev;
            $looper = 0;
            while ($loop_prev >= $old_order[$local_order - $local_offset + $looper] + 1) {
              db_query('
                UPDATE {deepsurvey_skip_responses}
                SET item_order = %d
                WHERE item_order = %d AND deepsurvey_nid=%d',
                $old_order[$local_order - $local_offset + $looper],
                $old_order[$local_order - $local_offset + $looper] + 1,
                $form_state['values']['nid']);
              $old_order[$local_order - $local_offset + $looper]++;
              $loop_prev = $old_order[$local_order - $local_offset + $looper];
              $looper++;
              $reset_flag = TRUE;
            }
          }
          $new_id = floor($local_prev +
            (($old_order[$local_order - $local_offset] - $local_prev) / 2));
          db_query('
            INSERT INTO {deepsurvey_skip_responses}
            (deepsurvey_nid, item_order, response_nid, operation,
            row_code, column_code, drop_down_code, response, target_nid)
            VALUES (%d, %d, %d, %d, %d, %d, %d, "%s", %d)',
            $form_state['values']['nid'], $new_id,
            $new_order[$local_order][0],
            $new_order[$local_order][1],
            $new_order[$local_order][2],
            $new_order[$local_order][3],
            $new_order[$local_order][4],
            $new_order[$local_order][5],
            $new_order[$local_order][6]);
          if ($new_order[$local_order][2] == '') {
            db_query('
            UPDATE {deepsurvey_skip_responses}
            SET row_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['nid']);
          }
          if ($new_order[$local_order][3] == '') {
            db_query('
            UPDATE {deepsurvey_skip_responses}
            SET column_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['nid']);
          }
          if ($new_order[$local_order][4] == '') {
            db_query('
            UPDATE {deepsurvey_skip_responses}
            SET drop_down_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['nid']);
          }
          $local_offset++;
          $local_prev = $new_id;
        }
        elseif ($old_order[$local_order - $local_offset] == FALSE && $new_order[$local_order] != FALSE) {
          if ($local_prev < DEEPSURVEY_MAX_ITEMS) {
            $new_id = floor($local_prev + ((DEEPSURVEY_MAX_ITEMS -
              $local_prev) / 2));
            db_query('
              INSERT INTO {deepsurvey_skip_responses}
              (deepsurvey_nid, item_order, response_nid, operation,
              row_code, column_code, drop_down_code, response, target_nid)
              VALUES (%d, %d, %d, %d, %d, %d, %d, "%s", %d)',
              $form_state['values']['nid'], $new_id,
              $new_order[$local_order][0],
              $new_order[$local_order][1],
              $new_order[$local_order][2],
              $new_order[$local_order][3],
              $new_order[$local_order][4],
              $new_order[$local_order][5],
              $new_order[$local_order][6]);
            if ($new_order[$local_order][2] == '') {
              db_query('
                UPDATE {deepsurvey_skip_responses}
                SET row_code = NULL
                WHERE item_order = %d AND deepsurvey_nid = %d',
                $new_id, $form_state['values']['nid']);
            }
            if ($new_order[$local_order][3] == '') {
              db_query('
                UPDATE {deepsurvey_skip_responses}
                SET column_code = NULL
                WHERE item_order = %d AND deepsurvey_nid = %d',
                $new_id, $form_state['values']['nid']);
            }
            if ($new_order[$local_order][4] == '') {
              db_query('
                UPDATE {deepsurvey_skip_responses}
                SET drop_down_code = NULL
                WHERE item_order = %d AND deepsurvey_nid = %d',
                $new_id, $form_state['values']['nid']);
            }
            $local_prev = $new_id;
          }
          else {
            $reset_flag = TRUE;
          }
        }
        else {
          $local_prev = $old_order[$local_order - $local_offset];
        }
        $local_order++;
      }

      if ($reset_flag) {
        _deepsurvey_reset_order('deepsurvey_skip_responses',
          $form_state['values']['nid']);
      }

    }
    drupal_set_message(t('Skip Updated'));
  }
  else {
    $form_state['redirect'] = $form_state['values']['last page'];
  }
}


/**
 * Validation function for the above.
 */
function deepsurvey_edit_validate_skip($form, &$form_state) {

  if ($form_state['clicked_button']['#value'] == t('Done')) {
    $number_of_items = 0;
    $highest_reorder = 0;
    // Item id of the reordered items.
    $reorder = array();
    reset($form_state['values']['table']);

    // Create a list of possible nid codes for questions.
    $question_list = array();
    $all_questions = db_query('SELECT nid, title
            FROM {node}
            WHERE type = "deepsurvey_question"');
    while ($line = db_fetch_object($all_questions)) {
      $question_list[$line->nid] = $line->title;
    }

    $survey_items_list = array();
    $all_survey_items = db_query('SELECT n.nid, n.title
      FROM {node} n
      INNER JOIN {deepsurvey_node_registration} r
      ON n.type = r.type');
    while ($line2 = db_fetch_object($all_survey_items)) {
      $survey_items_list[$line2->nid] = $line2->title;
    }

    while (list($key, $form_order) = each($form_state['values']['table'])) {

      if (is_null($question_list[$form_state['values']['table'][$key]['response title']]) && !is_null($form_state['values']['table'][$key]['response title'])) {
        form_set_error('table][' . $key . '][response title',
          t('The Response Question selected does not exist'));
      }

      if (is_null($survey_items_list[$form_state['values']['table'][$key]['target title']])) {
        form_set_error('table][' . $key . '][target title',
          t('The Target Item selected does not exist'));
      }

      if (($form_state['values']['table'][$key]['operation'] != DEEPSURVEY_SKIP_LT)
          && ($form_state['values']['table'][$key]['operation'] != DEEPSURVEY_SKIP_EQ)
          && ($form_state['values']['table'][$key]['operation'] != DEEPSURVEY_SKIP_GT)) {
        form_set_error('table][' . $key . '][operation',
          t('Invalid operation'));
      }

      if ((!is_numeric($form_state['values']['table'][$key]['row code'])) && $form_state['values']['table'][$nid]['row code'] != '') {
        form_set_error('table][' . $key . '][row code',
          t('Please enter a numeric row (x-axis) code only'));
      }

      if ((!is_numeric($form_state['values']['table'][$key]['col code'])) && $form_state['values']['table'][$nid]['col code'] != '') {
        form_set_error('table][' . $key . '][col code',
          t('Please enter a numeric column (y-axis) code only'));
      }

      if ((!is_numeric($form_state['values']['table'][$key]['drop down'])) && $form_state['values']['table'][$nid]['col code'] != '') {
        form_set_error('table][' . $key . '][drop down',
          t('Please enter a numeric drop down box code only'));
      }

      if (drupal_strlen($form_state['values']['table'][$key]['response']) > 65535) {
        form_set_error('question_text',
          t('The response text is too long - must be under 65535 letters and spaces'));
      }

      if ($form_state['values']['table'][$key]['remove box'] == TRUE && $form_state['values']['table'][$key]['reorder box'] != '') {
        form_set_error('table][' . $key . '][remove box',
          t('You can not reorder an item marked for deletion'));
      }

      if ($form_state['values']['table'][$key]['reorder box'] < 0 || $form_state['values']['table'][$key]['reorder box'] === 0) {
        form_set_error('table][' . $key . '][reorder box',
          t('reorder value 0 or less'));
      }
      elseif ($form_state['values']['table'][$key]['reorder box'] != '') {
        if ($reorder[$form_state['values']['table'][$key]['reorder box']] > 0) {
          form_set_error('table][' . $key . '][reorder box',
          t('duplicate reorder value'));
        }
        else {
          $reorder[$form_state['values']['table'][$key]['reorder box']] = $key;
        }
      }

      if ($form_state['values']['table'][$key]['reorder box'] > $highest_reorder) {
        $highest_reorder = $form_state['values']['table'][$key]['reorder box'];
      }

      $number_of_items++;
    }

    if ($highest_reorder > $number_of_items) {
      form_set_error('table][' . $reorder[$highest_reorder] . '][reorder box',
        t('reorder value higher than the number of items'));
    }

  }
}


/**
 * Build the form that allows the user to add new skips to a survey.
 *
 * @param array $form_state
 *   The form state array.
 *
 * @param int $deepsurvey_item_nid
 *   If $order is 0 then this is the node id of the survey object the node(s) is
 *   being added to, otherwise it is the item/survey relationship that the nodes
 *   are to be inserted after.
 *
 * @param int $order
 *   The order of the node before the new node(s) or the value of the next
 *   relationship id if order isn't 0.
 *
 * @param string $current_path
 *   The current path to this page, used to build links to the child nodes.
 *
 * @param string $last_page
 *   The path to the previous path for redirects, needed because of the
 *   limitations in drupal menu paths.
 *
 * @return array
 *   Form.
 */
function deepsurvey_skip_add_new_form($form_state, $deepsurvey_item_nid, $order, $current_path, $last_page) {

  $deepsurvey_title = db_fetch_object(db_query('
    SELECT title FROM {node}
    WHERE nid = %d', $deepsurvey_item_nid));

  drupal_set_title(t("Add new items to @name",
    array('@name' => check_plain($deepsurvey_title->title))));

  // Create a list of possible nid codes for response and target.
  $parent_node = substr(strrchr($last_page, "/"), 1);
  $current_question = db_fetch_object(db_query('
    SELECT type FROM {node}
    WHERE nid = %d', $parent_node));
  $parent_type = $current_question->type;
  $question_list = array('' => NULL);
  if ($parent_type == 'deepsurvey') {
    $all_questions = db_query('SELECT n.nid, n.title, r.item_order, n.type
      FROM {node} n
      INNER JOIN {deepsurvey_node_relationship} r
      ON n.nid = r.item_nid
      WHERE r.deepsurvey_nid = %d 
      ORDER BY r.item_order',
      $parent_node);
    $skip_order = db_fetch_object(db_query('
      SELECT item_order
      FROM {deepsurvey_node_relationship}
      WHERE item_nid = %d', $deepsurvey_item_nid));
  }
  elseif ($parent_type) {
    $all_questions = db_query('SELECT n.nid, n.title, r.item_order, n.type
      FROM {node} n
      INNER JOIN {deepsurvey_%s_node_relationship} r
      ON n.nid = r.item_nid
      WHERE r.deepsurvey_nid = %d
      ORDER BY r.item_order',
      $parent_type, $parent_node);
    $skip_order = db_fetch_object(db_query('
      SELECT item_order
      FROM {deepsurvey_%s_node_relationship}
      WHERE item_nid = %d', $parent_type, $deepsurvey_item_nid));
  }
  while ($current_question = db_fetch_object($all_questions)) {
    $question_list[$current_question->nid] = $current_question->title;
    if ($current_question->item_order < $skip_order->item_order && $current_question->type == 'deepsurvey_question') {
      $response_question_list[$current_question->nid] = $current_question->title;
    }
    elseif ($current_question->item_order > $skip_order->item_order) {
      $target_list[$current_question->nid] = $current_question->title;
    }
  }

  if (empty($target_list)) {
    $target_list = $question_list;
  }

  // Work around for AHAH bug.
  $form = array(
    '#cache' => TRUE,
    '#action' => url($current_path),
  );

  // Store parameters we are going to need when processing the form.
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $deepsurvey_item_nid,
  );
  $form['order'] = array(
    '#type' => 'value',
    '#value' => $order,
  );

  $form['table'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#theme' => 'deepsurvey_skip_add_new_skips_table',
    '#prefix' => '<div id="deepsurvey-skip-new">',
    '#suffix' => '</div>',
  );

  for ($response_loop = 1; $response_loop < 20; $response_loop++) {

    $row_list = array();
    $column_list = array();
    $drop_down_list = array();

    // Retrieve drop down box options based upon response question.
    if ($_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response title']) {
      // Row.
      $all_row = db_query('SELECT a.response, a.code
        FROM {deepsurvey_answer_list_responses} a
        INNER JOIN {deepsurvey_question_node_relationship} q
        ON a.deepsurvey_nid = q.item_nid
        WHERE q.deepsurvey_nid = %d AND q.axis = 1',
        $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response title']);
      while ($current_row = db_fetch_object($all_row)) {
        $row_list[$current_row->code] = $current_row->code . ': ' . $current_row->response;
      }

      // Column.
      $all_column = db_query('SELECT a.response, a.code
        FROM {deepsurvey_answer_list_responses} a
        INNER JOIN {deepsurvey_question_node_relationship} q
        ON a.deepsurvey_nid = q.item_nid
        WHERE q.deepsurvey_nid = %d AND q.axis = 2',
        $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response title']);
      while ($current_column = db_fetch_object($all_column)) {
        $column_list[$current_column->code] = $current_column->code . ': ' . $current_column->response;
      }

      // Drop down.
      $all_drop_down = db_query('SELECT a.response, a.code
        FROM {deepsurvey_answer_list_responses} a
        INNER JOIN {deepsurvey_question_node_relationship} q
        ON a.deepsurvey_nid = q.item_nid
        WHERE q.deepsurvey_nid = %d AND q.axis = 3',
        $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response title']);
      while ($current_drop_down = db_fetch_object($all_drop_down)) {
        $drop_down_list[$current_drop_down->code] = $current_drop_down->code . ': ' . $current_drop_down->response;
      }
    }

    $form['table'][$response_loop]['response title'] = array(
      '#type' => 'select',
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response title'],
      '#options' => array('' => '') + $response_question_list,
      '#ahah' => array(
        'path' => 'deepsurvey_skip_new_ahah',
        'wrapper' => 'deepsurvey-skip-new',
        'method' => 'replace',
        'event' => 'change',
      ),
    );
    $form['table'][$response_loop]['operation'] = array(
      '#type' => 'select',
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['operation'],
      '#options' => array(
        DEEPSURVEY_SKIP_LT => t('<'),
        DEEPSURVEY_SKIP_EQ => t('='),
        DEEPSURVEY_SKIP_GT => t('>'),
      ),
    );
    $form['table'][$response_loop]['row code'] = array(
      '#type' => 'select',
      '#maxlength' => 6,
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['row code'],
      '#options' => array('' => 'any') + $row_list,
      '#disabled' => empty($row_list),
    );
    $form['table'][$response_loop]['col code'] = array(
      '#type' => 'select',
      '#maxlength' => 6,
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['col code'],
      '#options' => array('' => 'any') + $column_list,
      '#disabled' => empty($column_list),
    );
    $form['table'][$response_loop]['drop down'] = array(
      '#type' => 'select',
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['drop down'],
      '#options' => array('' => 'any') + $drop_down_list,
      '#disabled' => empty($drop_down_list),
    );
    $form['table'][$response_loop]['response'] = array(
      '#type' => 'textfield',
      '#size' => 20,
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['response'],
    );
    $form['table'][$response_loop]['target title'] = array(
      '#type' => 'select',
      '#default_value' => $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid][$response_loop]['target title'],
      '#options' => array('' => '') + $target_list,
    );
  }
  $form['last page'] = array(
    '#type' => 'value',
    '#value' => $last_page . '/' . $deepsurvey_item_nid,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Done'),
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );

  // Unset AHAH values passed via session variables.
  $_SESSION['deepsurvey_skip_new_ahah'][$deepsurvey_item_nid] = NULL;

  return $form;
}

/**
 * Deepsurvey_skip_add_new submit function.
 */
function deepsurvey_skip_add_new_form_submit($form, &$form_state) {

  if ($form_state['clicked_button']['#value'] == t('Done') && is_null($form_state['post'])) {
    $local_last = $form_state['values']['order'];
    $next_item_in_order = db_fetch_object(db_query('
      SELECT item_order FROM {deepsurvey_skip_responses}
      WHERE deepsurvey_nid = %d AND item_order>%d
      ORDER BY item_order',
      $form_state['values']['rid'], $form_state['values']['order']));
    $local_next = $next_item_in_order->item_order;
    $reset_flag = FALSE;

    // In case survey has no items.
    if (!$local_next) {
      $local_next = DEEPSURVEY_MAX_ITEMS;
    }

    foreach (element_children($form_state['values']['table']) as $nid) {
      if ($form_state['values']['table'][$nid]['response title'] != 0) {
        if ($local_last >= $local_next + 1) {
          $loop_prev = $local_last;
          $loop_next = $local_next;
          $local_next++;
          while ($loop_prev >= $loop_next + 1) {
            db_query('
              UPDATE {deepsurvey_skip_responses}
              SET item_order = %d
              WHERE item_order = %d AND deepsurvey_nid = %d',
              $loop_next, $loop_next + 1, $form_state['values']['rid']);
            $loop_prev = $loop_next + 1;
            $loop_next = db_fetch_object(db_query('
              SELECT item_order
              FROM {deepsurvey_skip_responses}
              WHERE deepsurvey_nid = %d AND item_order>%d
              ORDER BY item_order',
              $form_state['values']['rid'], $loop_prev));
            $reset_flag = TRUE;
          }
        }
        $new_id = floor($local_last + (($local_next - $local_last) / 2));
        db_query('
          INSERT INTO {deepsurvey_skip_responses}
          (deepsurvey_nid, item_order, response_nid, operation,
          row_code, column_code, drop_down_code, response, target_nid)
          VALUES (%d, %d, %d, %d, %d, %d, %d, "%s", %d)',
          $form_state['values']['rid'], $new_id,
          $form_state['values']['table'][$nid]['response title'],
          $form_state['values']['table'][$nid]['operation'],
          $form_state['values']['table'][$nid]['row code'],
          $form_state['values']['table'][$nid]['col code'],
          $form_state['values']['table'][$nid]['drop down'],
          $form_state['values']['table'][$nid]['response'],
          $form_state['values']['table'][$nid]['target title']);
        if ($form_state['values']['table'][$nid]['row code'] == '') {
          db_query('
            UPDATE {deepsurvey_skip_responses}
            SET row_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['rid']);
        }
        if ($form_state['values']['table'][$nid]['col code'] == '') {
          db_query('
            UPDATE {deepsurvey_skip_responses}
            SET column_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['rid']);
        }
        if ($form_state['values']['table'][$nid]['drop down'] == '') {
          db_query('
            UPDATE {deepsurvey_skip_responses}
            SET drop_down_code = NULL
            WHERE item_order = %d AND deepsurvey_nid = %d',
            $new_id, $form_state['values']['rid']);
        }
        $local_last = $new_id;
      }
    }

    if ($reset_flag) {
      _deepsurvey_reset_order('deepsurvey_skip_responses',
        $form_state['values']['nid']);
    }

    drupal_set_message(t('New responses have been added to the skip'));
  }

  if (!is_null($form_state['post'])) {
    foreach (element_children($form_state['values']['table']) as $nid2) {
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['response title'] = $form_state['values']['table'][$nid2]['response title'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['operation'] = $form_state['values']['table'][$nid2]['operation'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['row code'] = $form_state['values']['table'][$nid2]['row code'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['col code'] = $form_state['values']['table'][$nid2]['col code'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['drop down'] = $form_state['values']['table'][$nid2]['drop down'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['response'] = $form_state['values']['table'][$nid2]['response'];
      $_SESSION['deepsurvey_skip_new_ahah'][$form_state['values']['rid']][$nid2]['target title'] = $form_state['values']['table'][$nid2]['target title'];
    }
  }
  else {
    $form_state['redirect'] = $form_state['values']['last page'];
  }

}

/**
 * Deepsurvey_skip_add_new validate function.
 */
function deepsurvey_skip_add_new_form_validate($form, &$form_state) {

  if ($form_state['clicked_button']['#value'] == t('Done') && is_null($form_state['post'])) {
    reset($form_state['values']['table']);

    // Create a list of possible nid codes for response and target.
    $question_list = array();
    $all_questions = db_query('SELECT nid, title
            FROM {node}
            WHERE type = "deepsurvey_question"');
    while ($line = db_fetch_object($all_questions)) {
      $question_list[$line->nid] = $line->title;
    }

    $survey_items_list = array();
    $all_survey_items = db_query('SELECT n.nid, n.title
      FROM {node} n
      INNER JOIN {deepsurvey_node_registration} r
      ON n.type = r.type');
    while ($line2 = db_fetch_object($all_survey_items)) {
      $survey_items_list[$line2->nid] = $line2->title;
    }

    foreach (element_children($form_state['values']['table']) as $nid) {

      if ($form_state['values']['table'][$nid]['response title'] != 0) {
        if (is_null($question_list[$form_state['values']['table'][$nid]['response title']])) {
          form_set_error('table][' . $nid . '][response title',
            t('The Response Question selected does not exist'));
        }

        if (is_null($survey_items_list[$form_state['values']['table'][$nid]['target title']])) {
          form_set_error('table][' . $nid . '][target title',
            t('The Target Item selected does not exist'));
        }

        if (($form_state['values']['table'][$nid]['operation'] != DEEPSURVEY_SKIP_LT)
            && ($form_state['values']['table'][$nid]['operation'] != DEEPSURVEY_SKIP_EQ)
            && ($form_state['values']['table'][$nid]['operation'] != DEEPSURVEY_SKIP_GT)) {
          form_set_error('table][' . $nid . '][operation',
            t('Invalid operation'));
        }

        if (!is_numeric($form_state['values']['table'][$nid]['row code']) && $form_state['values']['table'][$nid]['row code'] != '') {
          form_set_error('table][' . $nid . '][row code',
            t('Please enter a numeric code for each answer'));
        }

        if (!is_numeric($form_state['values']['table'][$nid]['col code']) && $form_state['values']['table'][$nid]['col code'] != '') {
          form_set_error('table][' . $nid . '][col code',
            t('Please enter a numeric code for each answer'));
        }

        if (!is_numeric($form_state['values']['table'][$nid]['drop down']) && $form_state['values']['table'][$nid]['drop down'] != '') {
          form_set_error('table][' . $nid . '][drop down',
            t('Please enter a numeric code for each answer'));
        }
      }

    }

  }
}

/**
 * Common code for handling AHAH callback.
 *
 * @return array
 *   Returns the recreated form after AHAH callback
 */
function deepsurvey_skip_ahah_utility_function() {

  // Preparing to retrieve the form from the cache.
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];

  // Retrieve the form from the cache.
  $form = form_get_cache($form_build_id, $form_state);

  // Preparing to Process the form.
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;

  // Process form.
  drupal_process_form($form_id, $form, $form_state);

  return drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);
}

/**
 * Callback handler for deepsurvey_skip_edit form AHAH.
 */
function deepsurvey_skip_edit_ahah() {
  $form = deepsurvey_skip_ahah_utility_function();

  $skip_form = $form['table'];
  unset($skip_form['#prefix'], $skip_form['#suffix']);
  $output = theme('status_messages') . drupal_render($skip_form);

  // Final rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * Callback handler for deepsurvey_skip_new form AHAH.
 */
function deepsurvey_skip_new_ahah() {
  $form = deepsurvey_skip_ahah_utility_function();

  $skip_form = $form['table'];
  unset($skip_form['#prefix'], $skip_form['#suffix']);
  $output = theme('status_messages') . drupal_render($skip_form);

  // Final rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * Removes deleted nodes of deepsurvey modules.
 *
 * Remove deleted deepsurvey content type node from tables in which they are
 * referenced. Called from other modules' hook_delete.
 *
 * @param int $deleted_node
 *   The node to be removed from tables.
 *
 * @return bool
 *   Return false if any problems were encountered otherwise true.
 */
function deepsurvey_skip_delete_content_type_node($deleted_node) {
  $rc1 = db_query('
    UPDATE {deepsurvey_skip_responses}
    SET response_nid = NULL
    WHERE response_nid = %d', $deleted_node);
  $rc2 = db_query('
    UPDATE {deepsurvey_skip_responses}
    SET target_nid = NULL
    WHERE target_nid = %d', $deleted_node);

  if ($rc1 && $rc2) {
    return TRUE;
  }
  else {
    return FALSE;
  }

}
